library row_date_picker;

import 'package:flutter/material.dart';

class RowDatePicker extends StatefulWidget {
  const RowDatePicker({
    final Key? key,
    required this.onTap,
    this.height = 150,
    this.type = DatePickerType.full,
    this.frameTextStyle = const TextStyle(color: Colors.black),
    this.unFrameTextStyle = const TextStyle(color: Colors.grey),
    this.backgroundColor = Colors.white,
    this.frameColor = Colors.grey,
  }) : super(key: key);

  /// on tap in frame
  final ValueChanged onTap;

  /// set height (Default: 200)
  final double height;

  /// show type DatePickerType(day,month,year,dayMonth,monthYear,full)
  final DatePickerType type;

  /// text style in frame
  final TextStyle frameTextStyle;

  /// text style in unframe
  final TextStyle unFrameTextStyle;

  /// background Color (Default: white)
  final Color backgroundColor;

  /// background Color (Default: grey)
  final Color frameColor;

  @override
  State<RowDatePicker> createState() => _RowDatePickerState();
}

class _RowDatePickerState extends State<RowDatePicker> {
  List month = [
    'January',
    'February',
    'March',
    'April',
    'May',
    'June',
    'July',
    'August',
    'September',
    'October',
    'November',
    'December',
  ];
  int _indexDay = DateTime.now().day + 1;
  int _indexMonth = DateTime.now().month;
  int _indexYear = 2;
  String selectedDay = '';
  String selectedMonth = '';
  String selectedYear = '';
  TextStyle frameStyle = const TextStyle(color: Colors.grey);
  @override
  void initState() {
    super.initState();
  }

  onTap() {
    selectedDay = (_indexDay + 1).toString();
    selectedMonth = month[_indexMonth];
    selectedYear = (DateTime.now().year - _indexYear).toString();
    switch (widget.type) {
      case DatePickerType.day:
        return widget.onTap(
          selectedDay,
        );
      case DatePickerType.month:
        return widget.onTap(
          selectedMonth,
        );
      case DatePickerType.year:
        return widget.onTap(
          selectedYear,
        );
      case DatePickerType.dayMonth:
        return widget.onTap(
          '$selectedDay, $selectedMonth',
        );
      case DatePickerType.monthYear:
        return widget.onTap(
          '$selectedMonth, $selectedYear',
        );
      case DatePickerType.full:
        return widget.onTap(
          '$selectedDay, $selectedMonth, $selectedYear',
        );

      default:
        return widget.onTap(
          '$selectedDay, $selectedMonth, $selectedYear',
        );
    }
  }

  @override
  Widget build(final BuildContext context) {
    return Container(
        color: widget.backgroundColor,
        height: widget.height,
        width: MediaQuery.of(context).size.width,
        child: Stack(
          children: [
            Positioned.fill(
              child: Align(
                child: InkWell(
                  onTap: () {
                    onTap();
                  },
                  child: Container(
                    height: 50,
                    width: MediaQuery.of(context).size.width,
                    decoration: BoxDecoration(
                      border: Border(
                        top: BorderSide(color: widget.frameColor),
                        bottom: BorderSide(color: widget.frameColor),
                      ),
                    ),
                  ),
                ),
              ),
            ),
            Row(
              mainAxisAlignment: MainAxisAlignment.center,
              children: [
                if (widget.type == DatePickerType.full ||
                    widget.type == DatePickerType.day ||
                    widget.type == DatePickerType.dayMonth)
                  SizedBox(
                    width: (MediaQuery.of(context).size.width / 4) * 1,
                    child: PageView.builder(
                      scrollDirection: Axis.vertical,
                      itemCount: 31,
                      controller: PageController(
                          viewportFraction: 0.3,
                          initialPage: DateTime.now().day + 1),
                      onPageChanged: (final int index) =>
                          setState(() => _indexDay = index),
                      itemBuilder: (final _, final i) {
                        return Transform.scale(
                          scale: i == _indexDay ? 1 : 0.9,
                          child: InkWell(
                            onTap: () {
                              if (i == _indexDay) {
                                onTap();
                              }
                            },
                            child: Center(
                              child: Text(
                                (i + 1).toString(),
                                style: i == _indexDay
                                    ? widget.frameTextStyle
                                    : widget.unFrameTextStyle,
                              ),
                            ),
                          ),
                        );
                      },
                    ),
                  ),
                if (widget.type == DatePickerType.full ||
                    widget.type == DatePickerType.month ||
                    widget.type == DatePickerType.dayMonth ||
                    widget.type == DatePickerType.monthYear)
                  SizedBox(
                    width: (MediaQuery.of(context).size.width / 4) * 1,
                    child: PageView.builder(
                      scrollDirection: Axis.vertical,
                      itemCount: 12,
                      controller: PageController(
                          viewportFraction: 0.3,
                          initialPage: DateTime.now().month),
                      onPageChanged: (final int index) =>
                          setState(() => _indexMonth = index),
                      itemBuilder: (final _, final i) {
                        return Transform.scale(
                          scale: i == _indexMonth ? 1 : 0.9,
                          child: InkWell(
                            onTap: () {
                              if (i == _indexMonth) {
                                onTap();
                              }
                            },
                            child: Center(
                              child: Text(
                                month[i],
                                style: i == _indexMonth
                                    ? widget.frameTextStyle
                                    : widget.unFrameTextStyle,
                              ),
                            ),
                          ),
                        );
                      },
                    ),
                  ),
                if (widget.type == DatePickerType.full ||
                    widget.type == DatePickerType.year ||
                    widget.type == DatePickerType.monthYear)
                  SizedBox(
                    width: (MediaQuery.of(context).size.width / 4) * 1,
                    child: PageView.builder(
                      scrollDirection: Axis.vertical,
                      itemCount: 300,
                      reverse: true,
                      controller:
                          PageController(viewportFraction: 0.3, initialPage: 2),
                      onPageChanged: (final int index) =>
                          setState(() => _indexYear = index),
                      itemBuilder: (final _, final i) {
                        return Transform.scale(
                          scale: i == _indexYear ? 1 : 0.9,
                          child: InkWell(
                            onTap: () {
                              if (i == _indexYear) {
                                onTap();
                              }
                            },
                            child: Center(
                              child: Text(
                                '${DateTime.now().year - i}',
                                style: i == _indexYear
                                    ? widget.frameTextStyle
                                    : widget.unFrameTextStyle,
                              ),
                            ),
                          ),
                        );
                      },
                    ),
                  ),
              ],
            )
          ],
        ));
  }
}

enum DatePickerType {
  day,
  month,
  year,
  dayMonth,
  monthYear,
  full,
}
